package com.dwl.mindoc.service;

import com.dwl.mindoc.MinConfig;
import com.dwl.mindoc.dao.BaseDao;
import com.dwl.mindoc.database.BaseFactory;
import com.dwl.mindoc.database.Database;
import com.dwl.mindoc.domain.ColumnVo;
import com.dwl.mindoc.domain.TableVo;
import com.itextpdf.kernel.color.DeviceRgb;
import com.itextpdf.kernel.events.PdfDocumentEvent;
import com.itextpdf.kernel.font.PdfFont;
import com.itextpdf.kernel.font.PdfFontFactory;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfWriter;
import com.itextpdf.layout.element.Paragraph;
import com.itextpdf.layout.element.Table;
import com.itextpdf.layout.element.Cell;
import com.itextpdf.layout.property.HorizontalAlignment;
import com.itextpdf.layout.property.TextAlignment;
import com.itextpdf.layout.property.UnitValue;
import com.itextpdf.layout.Document;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ResourceUtils;

import javax.annotation.PostConstruct;
import java.io.*;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @program: mindoc
 * @description: generator
 * @author: daiwenlong
 * @create: 2018-10-13 13:00
 **/
@Service
public class GenerateService {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DBGenerDocService factory;

    @Autowired
    private BaseDao dao;

    @Autowired
    private MinConfig config;

    @PostConstruct
    public void genertator() {
        try {
            Database base = factory.getDataBase();
            List<TableVo> tables = dao.getTables(base);
            tables.forEach(table -> {
                table.setColumns(dao.getColumns(base, table.getTableName()));
                logger.info("mindoc - TableName：{} TableComment：{} loading...", table.getTableName(),
                        table.getTableComment());
            });
            if ("pdf".equalsIgnoreCase(config.getValue("fileType"))) {
                makePdf(tables);
            } else {
                makeDoc(tables);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 文件名
     */
    @Value("${baseName}")
    private String fileName;

    /**
     * 生成doc
     *
     * @param tables
     * @return
     * @throws IOException
     * @throws TemplateException
     */
    private void makeDoc(List<TableVo> tables) {
        logger.info("mindoc - makeDoc satrting...");
        // 第一步：创建一个Configuration对象。
        Configuration configuration = new Configuration(Configuration.getVersion());
        // 第二步：设置模板文件所在的路径。
        try {
            configuration.setDirectoryForTemplateLoading(ResourceUtils.getFile("classpath:"));
            // 第三步：设置模板文件使用的字符集。
            configuration.setDefaultEncoding("utf-8");
            // 第四步：加载一个模板，创建一个模板对象。
            Template template = configuration.getTemplate("doc.xml");
            // 第五步：创建一个模板使用的数据集。
            Map<String, Object> dataModel = new HashMap<>();
            dataModel.put("tables", tables);
            // 生成文件放在项目根目录下
            File outFile = new File(System.getProperty("user.dir") + "\\" + fileName + ".doc");
            Writer out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outFile), "UTF-8"));
            // 第六步：调用模板对象的process方法输出文件。
            template.process(dataModel, out);
            logger.info("mindoc - MakeDoc succeeded.");
            logger.info("mindoc - Doc directory {}", outFile);
        } catch (IOException | TemplateException e) {
            logger.warn("mindoc - MakeDoc failed.");
        }
    }

    private void makePdf(List<TableVo> tables) {

        try {
            logger.info("mindoc - makePdf satrting...");
            PdfDocument pdfDoc = new PdfDocument(
                    new PdfWriter(System.getProperty("user.dir") + "\\" + fileName + ".pdf"));
            Document doc = new Document(pdfDoc);// 构建文档对象
            TextFooterEventHandler eh = new TextFooterEventHandler(doc);
            pdfDoc.addEventHandler(PdfDocumentEvent.END_PAGE, eh);
            // 中文字体
            PdfFont sysFont = PdfFontFactory.createFont("STSongStd-Light", "UniGB-UCS2-H", false);
            Paragraph paragraph = new Paragraph();
            paragraph.add("数据库设计文档").setFont(sysFont).setBold().setFontSize(20).setTextAlignment(TextAlignment.CENTER);
            doc.add(paragraph);
            int num = 0;
            for (TableVo vo : tables) {
                num++;
                doc.add(new Paragraph(""));
                String title = num + "  表名：" + vo.getTableName() + "   表注释：" + vo.getTableComment();
                doc.add(new Paragraph(title).setFont(sysFont).setBold());
                // 构建表格以100%的宽度
                Table table = new Table(5).setWidth(UnitValue.createPercentValue(100));

                table.addCell(new Cell().add(new Paragraph("列名")).setFont(sysFont)
                        .setBackgroundColor(new DeviceRgb(221, 234, 238)));
                table.addCell(new Cell().add(new Paragraph("数据类型")).setFont(sysFont)
                        .setBackgroundColor(new DeviceRgb(221, 234, 238)));
                table.addCell(new Cell().add(new Paragraph("字段长度")).setFont(sysFont)
                        .setBackgroundColor(new DeviceRgb(221, 234, 238)));
                table.addCell(new Cell().add(new Paragraph("允许空")).setFont(sysFont)
                        .setBackgroundColor(new DeviceRgb(221, 234, 238)));
                table.addCell(new Cell().add(new Paragraph("备注")).setFont(sysFont)
                        .setBackgroundColor(new DeviceRgb(221, 234, 238)));
                int j = 0;
                for (ColumnVo col : vo.getColumns()) {
                    j++;
                    String columnName = ifNull(col.getColumnName());
                    String columnType =ifNull(col.getColumnType());
                    String columnKey = ifNull(col.getColumnKey());
                    String isNullable = ifNull(col.getIsNullable());
                    String columnComment = ifNull(col.getColumnComment());
                    table.addCell(new Cell().add(new Paragraph(columnName)).setFont(sysFont));
                    table.addCell(new Cell().add(new Paragraph(columnType)).setFont(sysFont));
                    table.addCell(new Cell().add(new Paragraph(columnKey)).setFont(sysFont));
                    table.addCell(new Cell().add(new Paragraph(isNullable)).setFont(sysFont));
                    table.addCell(new Cell().add(new Paragraph(columnComment)).setFont(sysFont));
                }
                // 将表格添加入文档并页面居中
                doc.add(table.setHorizontalAlignment(HorizontalAlignment.CENTER));
            }
            doc.close();
            logger.info("mindoc - MakePdf succeeded.");
            logger.info("mindoc - Pdf directory {}", System.getProperty("user.dir") + "\\" + fileName + ".pdf");
        } catch (Exception e) {
            logger.warn("mindoc - MakePdf failed.");
            System.err.println(e.getMessage());
        }
    }

    private String ifNull(String str){
        if(null==str){
            str ="";
        }
        return  str;
    }

}
